<?php
    require 'vendor/autoload.php';
    
    use Google\Cloud\Speech\V1\SpeechClient;
    use Google\Cloud\Speech\V1\RecognitionAudio;
    use Google\Cloud\Speech\V1\RecognitionConfig;
    use Google\Cloud\Speech\V1\RecognitionConfig\AudioEncoding;
    
    putenv('GOOGLE_APPLICATION_CREDENTIALS=' . __DIR__ . '/mediamonitor-455121-be3eb805f1d7.json');

    /**
     * @param string $audioFile path to an audio file
     */
    transcribe_auto_punctuation('./recordings/radio_stream_20250330_070003.mp3');
    function transcribe_auto_punctuation(string $audioFile)
    {
        // change these variables if necessary
        $encoding = AudioEncoding::MP3;
        $sampleRateHertz = 48000;
        $languageCode = 'en-US';
    
        // get contents of a file into a string
        $content = file_get_contents($audioFile);
    
        // set string as audio content
        $audio = (new RecognitionAudio())
            ->setContent($content);
    
        // set config
        $config = (new RecognitionConfig())
            ->setEncoding($encoding)
            ->setSampleRateHertz($sampleRateHertz)
            ->setLanguageCode($languageCode)
            ->setEnableAutomaticPunctuation(true);
    
        // create the speech client
        $client = new SpeechClient([
            'credentials' => json_decode(file_get_contents(getenv('GOOGLE_APPLICATION_CREDENTIALS')), true),
        ]);
    
        // make the API call
        try {
            $response = $client->recognize($config, $audio);
            $results = $response->getResults();
        } catch (Exception $e) {
            echo 'Error: ' . $e->getMessage();
            exit;
        }
        // $response = $client->recognize($config, $audio);
        // $results = $response->getResults();
    
        // print results
        foreach ($results as $result) {
            $alternatives = $result->getAlternatives();
            $mostLikely = $alternatives[0];
            $transcript = $mostLikely->getTranscript();
            $confidence = $mostLikely->getConfidence();
            printf('Transcript: %s' . PHP_EOL, $transcript);
            printf('Confidence: %s' . PHP_EOL, $confidence);
        }
    
        $client->close();
    }
?>