<?php
ini_set('display_errors', '1');
ini_set('display_startup_errors', '1');
ini_set('memory_limit', '512M');
set_time_limit(0);
error_reporting(E_ALL);
require_once 'vendor/autoload.php';

// use Google\Cloud\Speech\V2\SpeechClient;
use Google\Cloud\Speech\V2\Client\SpeechClient;
use Google\Cloud\Speech\V2\StreamingRecognizeRequest;
use Google\Cloud\Speech\V2\StreamingRecognitionConfig;
use Google\Cloud\Speech\V2\RecognitionConfig;
use Google\Cloud\Speech\V2\AutoDetectDecodingConfig;

// Configuration
$projectId = "mediamonitor-455121";
$streamUrl = "https://npr-ice.streamguys1.com/live.mp3";
putenv('GOOGLE_APPLICATION_CREDENTIALS=' . __DIR__ . '/mediamonitor-455121-be3eb805f1d7.json');

// Database setup (optional)
$pdo = new PDO('mysql:host=localhost;dbname=urbex_developers', 'urbex_developers', 'kolkunzadeyeye1');
$pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);


// Audio parameters
$sampleRate = 16000; // Hz
$bytesPerSample = 2; // 16-bit = 2 bytes
$bytesPerSecond = $sampleRate * $bytesPerSample;
$maxChunkSize = 25600; // Google's maximum chunk size (25,600 bytes)
$chunkDuration = $maxChunkSize / $bytesPerSecond; // 0.8 seconds

// Permanent storage directory structure
$storageDir = __DIR__ . '/audio_archive/' . date('Y/m/d');
if (!file_exists($storageDir)) {
    mkdir($storageDir, 0777, true); // Create directory with full permissions
}

// Current audio file (rotates hourly)
$currentHour = date('H');
$audioFile = $storageDir . '/stream_' . date('Y-m-d_H') . '00.raw';

// Initialize Speech Client
$client = new SpeechClient([
    'projectId' => $projectId,
    'credentials' => json_decode(file_get_contents(getenv('GOOGLE_APPLICATION_CREDENTIALS')), true)
]);


$ffmpegCmd = sprintf(
    'ffmpeg -i %s -ac 1 -ar %d -f s16le -y %s',
    escapeshellarg($audioFile),
    $sampleRate,
    escapeshellarg($storageDir . '/for_transcription.raw')
);
// FFmpeg command with permanent storage
// $ffmpegCmd = sprintf(
//     'ffmpeg -re -i %s -acodec pcm_s16le -ac 1 -ar %d -fflags +genpts -flags low_delay -f s16le -y %s 2>%s',
//     escapeshellarg($streamUrl),
//     $sampleRate,
//     escapeshellarg($audioFile),
//     escapeshellarg($storageDir . '/ffmpeg.log')
// );

$ffmpegPid = exec("$ffmpegCmd > /dev/null & echo $!");

// Configure speech recognition
$recognitionConfig = (new RecognitionConfig())
    ->setAutoDecodingConfig(new AutoDetectDecodingConfig())
    ->setLanguageCodes(['en-US'], ['en-UK'])
    ->setModel('long');

$streamingConfig = (new StreamingRecognitionConfig())
    ->setConfig($recognitionConfig);

$configRequest = (new StreamingRecognizeRequest())
    ->setRecognizer("projects/$projectId/locations/global/recognizers/_")
    ->setStreamingConfig($streamingConfig);

// Main processing loop
$maxRetries = 3;
$retryCount = 0;
$streamActive = false;
$processedBytes = 0;
$lastHourChecked = $currentHour;

while ($retryCount < $maxRetries) {
    try {
        // Initialize streaming connection
        $stream = $client->streamingRecognize();
        $stream->write($configRequest);
        $streamActive = true;
        
        while (true) {
            // Check if we need to rotate to a new file (hourly)
            $currentHour = date('H');
            if ($currentHour != $lastHourChecked) {
                $lastHourChecked = $currentHour;
                $newAudioFile = $storageDir . '/stream_' . date('Y-m-d_H') . '00.raw';
                
                // Restart FFmpeg with new file
                exec("kill $ffmpegPid 2>/dev/null");
                // $ffmpegCmd = "ffmpeg -i \"$streamUrl\" -ar 16000 \"$newAudioFile\"";
                $ffmpegCmd = sprintf(
                    'ffmpeg -re -i %s -acodec pcm_s16le -ac 1 -ar %d -fflags +genpts -flags low_delay -f s16le -y %s 2>>%s',
                    escapeshellarg($streamUrl),
                    $sampleRate,
                    escapeshellarg($newAudioFile),
                    escapeshellarg($storageDir . '/ffmpeg.log')
                );
                $ffmpegPid = exec("$ffmpegCmd > /dev/null & echo $!");
                $audioFile = $newAudioFile;
                $processedBytes = 0;
            }
            
            // Check for new audio data
            clearstatcache(true, $audioFile);
            $currentSize = file_exists($audioFile) ? filesize($audioFile) : 0;
            
            if ($currentSize - $processedBytes >= $maxChunkSize) {
                // Read exactly one compliant chunk
                $fp = fopen($audioFile, 'rb');
                fseek($fp, $processedBytes);
                $chunk = fread($fp, $maxChunkSize);
                fclose($fp);
                
                if (!empty($chunk)) {
                    $request = (new StreamingRecognizeRequest())
                        ->setAudio($chunk);
                    $stream->write($request);
                    $processedBytes += strlen($chunk);
                    
                    // Output progress
                    $secondsProcessed = $processedBytes / $bytesPerSecond;
                    echo "[" . date('Y-m-d H:i:s') . "] Processed: " . 
                         round($secondsProcessed, 1) . "s of current file\n";
                }
            }
            
            // Read responses
            $responses = $stream->read();
            foreach ($responses as $response) {
                foreach ($response->getResults() as $result) {
                    if ($result->getAlternatives()) {
                        $transcript = $result->getAlternatives()[0]->getTranscript();
                        $confidence = $result->getAlternatives()[0]->getConfidence();
                        
                        // Optional: Store in database
                        if (isset($pdo)) {
                            $stmt = $pdo->prepare(
                                "INSERT INTO transcriptions (transcript) 
                                VALUES (:transcript)"
                            );
                            $stmt->execute([
                                'transcript' => $transcript
                            ]);
                        }
                        
                        // Save transcript with timestamp
                        $logEntry = sprintf(
                            "[%s] %.1fs: %s (Confidence: %.2f)\n",
                            date('Y-m-d H:i:s'),
                            $processedBytes / $bytesPerSecond,
                            trim($transcript),
                            $confidence
                        );
                        
                        file_put_contents(
                            $storageDir . '/transcripts.log',
                            $logEntry,
                            FILE_APPEND
                        );
                        
                        echo $logEntry;
                    }
                }
            }
            
            usleep(100000); // 100ms delay between checks
        }
        
    } catch (Exception $e) {
        print_r("Error: " . $e->getMessage());
        $retryCount++;
        $streamActive = false;
        
        if (isset($stream) && $streamActive) {
            try {
                $stream->closeWrite();
            } catch (Exception $e) {
                error_log("Stream close error: " . $e->getMessage());
            }
        }
        
        sleep(5); // Wait before retrying
    }
}

// Cleanup
if (isset($ffmpegPid)) {
    exec("kill $ffmpegPid 2>/dev/null");
}

$client->close();
?>