<?php
ini_set('display_errors', '1');
ini_set('display_startup_errors', '1');
ini_set('memory_limit', '512M');
set_time_limit(0);
error_reporting(E_ALL);
require 'vendor/autoload.php';

use Google\Cloud\Speech\V2\Client\SpeechClient;
use Google\Cloud\Speech\V2\StreamingRecognizeRequest;
use Google\Cloud\Speech\V2\StreamingRecognitionConfig;
use Google\Cloud\Speech\V2\RecognitionConfig;
use Google\Cloud\Speech\V2\AutoDetectDecodingConfig;
use Google\ApiCore\ApiException;

// Database connection
$conn = new mysqli('localhost', "urbex_developers", "kolkunzadeyeye1", "urbex_developers");
if ($conn->connect_error) {
    die("Connection failed: " . $conn->connect_error);
}

// Google Cloud settings
$projectId = "mediamonitor-455121";
putenv('GOOGLE_APPLICATION_CREDENTIALS=' . __DIR__ . '/mediamonitor-455121-be3eb805f1d7.json');

// Define your streams here
$streams = [
    [
        'url' => 'https://npr-ice.streamguys1.com/live.mp3',
        'name' => 'NPR',
        'server' => 'LOS-SVR-1',
        'state' => 'LAGOS'
    ],
    // Add more streams as needed
];

// Function to initialize speech client with retries
function initializeSpeechClient($maxRetries = 3) {
    $retryCount = 0;
    $lastError = null;
    
    while ($retryCount < $maxRetries) {
        try {
            return new SpeechClient([
                'transport' => 'grpc',
                'credentials' => json_decode(file_get_contents(getenv('GOOGLE_APPLICATION_CREDENTIALS')), true)
            ]);
        } catch (Exception $e) {
            $lastError = $e;
            $retryCount++;
            sleep(5 * $retryCount); // Exponential backoff
        }
    }
    
    throw new Exception("Failed to initialize SpeechClient after $maxRetries attempts: " . $lastError->getMessage());
}

// Function to start streaming recognition with retries
function startStreamingRecognition($client, $projectId, $maxRetries = 3) {
    $retryCount = 0;
    $lastError = null;
    
    while ($retryCount < $maxRetries) {
        try {
            // Set up recognition config
            $recognitionConfig = (new RecognitionConfig())
                ->setAutoDecodingConfig(new AutoDetectDecodingConfig())
                ->setLanguageCodes(['en-US', 'en-UK'])
                ->setModel('latest_long');
                
            $streamConfig = (new StreamingRecognitionConfig())
                ->setConfig($recognitionConfig);
                
            $configRequest = (new StreamingRecognizeRequest())
                ->setRecognizer("projects/$projectId/locations/global/recognizers/_")
                ->setStreamingConfig($streamConfig);
                
            $stream = $client->streamingRecognize();
            $stream->write($configRequest);
            
            return $stream;
        } catch (ApiException $e) {
            $lastError = $e;
            $retryCount++;
            
            // Specific handling for quota exceeded errors
            if (strpos($e->getMessage(), 'RESOURCE_EXHAUSTED') !== false) {
                sleep(30); // Longer wait for quota issues
            } else {
                sleep(5 * $retryCount); // Exponential backoff
            }
        } catch (Exception $e) {
            $lastError = $e;
            $retryCount++;
            sleep(5 * $retryCount);
        }
    }
    
    throw new Exception("Failed to start streaming recognition after $maxRetries attempts: " . $lastError->getMessage());
}

// Function to process a single stream
function processStream($streamConfig, $dbConn) {
    $projectId = $GLOBALS['projectId'];
    
    try {
        $client = initializeSpeechClient();
        
        while (true) {
            echo "Starting new session for {$streamConfig['name']}...\n";
            
            // Create storage directory
            $storageDir = __DIR__ . '/audio_archive/' . date('Y/m/d');
            if (!file_exists($storageDir)) {
                mkdir($storageDir, 0777, true);
            }
            
            // Generate unique filename
            $transcription_id = rand(10000, 99999) . uniqid() . time() . '.wav';
            $audioFile = $storageDir . '/' . $transcription_id;
            
            // Get stream metadata
            $streamInfo = @shell_exec("ffprobe \"{$streamConfig['url']}\" -show_format -show_streams -print_format json 2>&1");
            $metadata = @json_decode($streamInfo, true);
            
            $station_name = $metadata['format']['tags']['icy-name'] ?? $streamConfig['name'];
            $programme_name = $metadata['format']['tags']['StreamTitle'] ?? 'Unknown Programme';
            $station_url = $metadata['format']['filename'] ?? $streamConfig['url'];
            
            // Set up FFmpeg process
            $descriptors = [
                0 => ["pipe", "r"],  // STDIN
                1 => ["pipe", "w"],  // STDOUT (audio)
                2 => ["pipe", "w"]   // STDERR
            ];
            
            $ffmpegCommand = "ffmpeg -user_agent \"FFmpeg\" -re -i \"{$streamConfig['url']}\" " .
                             "-acodec pcm_s16le -ac 1 -ar 16000 -map 0:a -f tee \"[f=wav]pipe:1|[f=wav]$audioFile\"";
            
            $process = proc_open($ffmpegCommand, $descriptors, $pipes);
            
            if (!is_resource($process)) {
                echo "Failed to open FFmpeg stream for {$streamConfig['name']}\n";
                sleep(10);
                continue;
            }
            
            // Close unused pipes
            fclose($pipes[0]);
            fclose($pipes[2]);
            
            try {
                $stream = startStreamingRecognition($client, $projectId);
                $startTime = time();
                $lastAudioTime = time();
                
                // Process audio stream
                while (time() - $startTime < 10) { // Restart before 5-minute limit
                    $chunk = fread($pipes[1], 4096);
                    
                    if ($chunk !== false && strlen($chunk) > 0) {
                        $lastAudioTime = time();
                        try {
                            $request = (new StreamingRecognizeRequest())->setAudio($chunk);
                            $stream->write($request);
                        } catch (Exception $e) {
                            echo "Error writing to stream: " . $e->getMessage() . "\n";
                            break;
                        }
                    } else {
                        // Check if process is still running
                        $status = proc_get_status($process);
                        if (!$status['running']) {
                            echo "FFmpeg process died for {$streamConfig['name']}\n";
                            break;
                        }
                        
                        // Check if we haven't received audio for too long
                        if (time() - $lastAudioTime > 30) {
                            echo "No audio received for 30 seconds for {$streamConfig['name']}\n";
                            break;
                        }
                        
                        usleep(10000); // Small delay to prevent busy waiting
                    }
                }
                
                // Get transcription results
                try {
                    foreach ($stream->closeWriteAndReadAll() as $response) {
                        foreach ($response->getResults() as $result) {
                            foreach ($result->getAlternatives() as $alternative) {
                                $trans = $dbConn->real_escape_string($alternative->getTranscript());
                                $time = time();
                                $query = "INSERT INTO transcriptions (audio_id, transcript, station_name, programme_name, " .
                                         "programme_url, state, streaming_server, time) VALUES " .
                                         "('$transcription_id', '$trans', '$station_name', '$programme_name', " .
                                         "'$station_url', '{$streamConfig['state']}', '{$streamConfig['server']}', '$time')";
                                $dbConn->query($query);
                            }
                        }
                    }
                } catch (Exception $e) {
                    echo "Error processing transcription: " . $e->getMessage() . "\n";
                }
            } catch (Exception $e) {
                echo "Error with speech recognition stream: " . $e->getMessage() . "\n";
            }
            
            // Clean up
            @proc_terminate($process);
            @fclose($pipes[1]);
            @proc_close($process);
            
            echo "Restarting stream for {$streamConfig['name']} after 5 minutes...\n";
            sleep(1); // Brief pause before restarting
        }
    } catch (Exception $e) {
        echo "Fatal error for stream {$streamConfig['name']}: " . $e->getMessage() . "\n";
    }
}

// Fork processes for each stream
$children = [];
foreach ($streams as $stream) {
    $pid = pcntl_fork();
    
    if ($pid == -1) {
        die("Could not fork for stream {$stream['name']}");
    } elseif ($pid) {
        // Parent process - store child PID
        $children[] = $pid;
    } else {
        // Child process - handle the stream
        processStream($stream, $conn);
        exit(); // Child process should exit when done
    }
}

// Parent process waits for children
foreach ($children as $pid) {
    pcntl_waitpid($pid, $status);
}

$conn->close();
?>